@@ -23,6 +23,8 @@ import (
 	"errors"
 	"fmt"
 	"net/url"
+	"slices"
+	"strings"
 
 	"github.com/go-logr/logr"
 	corev1 "k8s.io/api/core/v1"
@@ -132,6 +134,9 @@ func TLSConfigFromSecret(ctx context.Context, secret *corev1.Secret, opts ...TLS
 // The function expects the secret to contain an "address" field with the
 // proxy URL. Optional "username" and "password" fields can be provided
 // for proxy authentication.
+//
+// Supported proxy schemes are: http, https, and socks5.
+// The proxy URL must not exceed 2048 characters.
 func ProxyURLFromSecret(ctx context.Context, secret *corev1.Secret) (*url.URL, error) {
 	addressData, exists := secret.Data[KeyAddress]
 	if !exists {
@@ -144,10 +149,10 @@ func ProxyURLFromSecret(ctx context.Context, secret *corev1.Secret) (*url.URL, e
 		return nil, fmt.Errorf("secret '%s': proxy address is empty", ref)
 	}
 
-	proxyURL, err := url.Parse(address)
+	proxyURL, err := parseProxyURL(address)
 	if err != nil {
 		ref := client.ObjectKeyFromObject(secret)
-		return nil, fmt.Errorf("secret '%s': failed to parse proxy address '%s': %w", ref, address, err)
+		return nil, fmt.Errorf("secret '%s': %w", ref, err)
 	}
 
 	username, hasUsername := secret.Data[KeyUsername]
@@ -162,6 +167,39 @@ func ProxyURLFromSecret(ctx context.Context, secret *corev1.Secret) (*url.URL, e
 	return proxyURL, nil
 }
 
+func parseProxyURL(address string) (*url.URL, error) {
+	if err := validateProxyURLString(address); err != nil {
+		return nil, err
+	}
+
+	u, err := url.Parse(address)
+	if err != nil {
+		return nil, fmt.Errorf("failed to parse proxy address '%s': %w", address, err)
+	}
+
+	if err := validateProxyURLStruct(u); err != nil {
+		return nil, err
+	}
+
+	return u, nil
+}
+
+func validateProxyURLString(address string) error {
+	if len(address) > MaxProxyURLLength {
+		return fmt.Errorf("proxy URL exceeds maximum length of %d characters", MaxProxyURLLength)
+	}
+	return nil
+}
+
+func validateProxyURLStruct(u *url.URL) error {
+	if !slices.Contains(supportedProxySchemes, u.Scheme) {
+		supportedSchemes := strings.Join(supportedProxySchemes, ", ")
+		return fmt.Errorf("proxy URL must use one of the supported schemes (%s), got '%s'",
+			supportedSchemes, u.Scheme)
+	}
+	return nil
+}
+
 // BasicAuthFromSecret retrieves basic authentication credentials from a Kubernetes secret.
 //
 // The function expects the secret to contain "username" and "password" fields.
@@ -260,17 +298,18 @@ func SSHAuthFromSecret(ctx context.Context, secret *corev1.Secret) (*SSHAuth, er
 
 // GitHubAppDataFromSecret retrieves GitHub App authentication data from a Kubernetes secret.
 //
-// The function expects the secret to contain "githubAppID", "githubAppInstallationID", and
-// "githubAppPrivateKey" fields. All three fields are required and the function will return
-// an error if any is missing. Optional "githubAppBaseURL" field can be present for GitHub
-// Enterprise Server instances.
+// The function expects the secret to contain "githubAppID" and "githubAppPrivateKey",
+// and exactly one of "githubAppInstallationOwner" or "githubAppInstallationID" fields.
+// All three fields are required and the function will return an error if any is missing.
+// Optional "githubAppBaseURL" field can be present for GitHub Enterprise Server instances.
 func GitHubAppDataFromSecret(ctx context.Context, secret *corev1.Secret) (GitHubAppData, error) {
 	_, hasAppID := secret.Data[KeyGitHubAppID]
+	_, hasInstallationOwner := secret.Data[KeyGitHubAppInstallationOwner]
 	_, hasInstallationID := secret.Data[KeyGitHubAppInstallationID]
 	_, hasPrivateKey := secret.Data[KeyGitHubAppPrivateKey]
 
 	// Complete absence - return KeyNotFoundError (will be ignored by trySetAuth)
-	if !hasAppID && !hasInstallationID && !hasPrivateKey {
+	if !hasAppID && !hasInstallationOwner && !hasInstallationID && !hasPrivateKey {
 		return nil, &KeyNotFoundError{Key: KeyGitHubAppID, Secret: secret}
 	}
 
@@ -278,17 +317,25 @@ func GitHubAppDataFromSecret(ctx context.Context, secret *corev1.Secret) (GitHub
 	if !hasAppID {
 		return nil, &KeyNotFoundError{Key: KeyGitHubAppID, Secret: secret}
 	}
-	if !hasInstallationID {
-		return nil, &KeyNotFoundError{Key: KeyGitHubAppInstallationID, Secret: secret}
-	}
 	if !hasPrivateKey {
 		return nil, &KeyNotFoundError{Key: KeyGitHubAppPrivateKey, Secret: secret}
 	}
+	if hasInstallationOwner == hasInstallationID {
+		// Either both are present or both are missing - both cases are errors
+		return nil, fmt.Errorf("secret '%s' must contain exactly one of '%s' or '%s'",
+			client.ObjectKeyFromObject(secret), KeyGitHubAppInstallationOwner, KeyGitHubAppInstallationID)
+	}
 
 	data := GitHubAppData{
-		KeyGitHubAppID:             secret.Data[KeyGitHubAppID],
-		KeyGitHubAppInstallationID: secret.Data[KeyGitHubAppInstallationID],
-		KeyGitHubAppPrivateKey:     secret.Data[KeyGitHubAppPrivateKey],
+		KeyGitHubAppID:         secret.Data[KeyGitHubAppID],
+		KeyGitHubAppPrivateKey: secret.Data[KeyGitHubAppPrivateKey],
+	}
+
+	if owner, exists := secret.Data[KeyGitHubAppInstallationOwner]; exists {
+		data[KeyGitHubAppInstallationOwner] = owner
+	}
+	if installationID, exists := secret.Data[KeyGitHubAppInstallationID]; exists {
+		data[KeyGitHubAppInstallationID] = installationID
 	}
 
 	if baseURLData, exists := secret.Data[KeyGitHubAppBaseURL]; exists {
